@@ -208,7 +208,7 @@ void demux_Open( void )
  *****************************************************************************/
 void demux_Close( void )
 {
-    int i, j;
+    int i;
 
     psi_table_free( pp_current_pat_sections );
     psi_table_free( pp_next_pat_sections );
@@ -222,10 +222,7 @@ void demux_Close( void )
     for ( i = 0; i < MAX_PIDS; i++ )
     {
         free( p_pids[i].p_psi_buffer );
-        for ( j = 0; j < p_pids[i].i_nb_outputs; j++ )
-        {
-            free( p_pids[i].pp_outputs );
-        }
+        free( p_pids[i].pp_outputs );
     }
 
     for ( i = 0; i < i_nb_sids; i++ )
@@ -1175,7 +1172,7 @@ static void CopyDescriptors( uint8_t *p_descs, uint8_t *p_current_descs )
         uint8_t i_tag = desc_get_tag( p_current_desc );
 
         j++;
-        if ( b_enable_ecm && i_tag == 0x9 ) continue;
+        if ( !b_enable_ecm && i_tag == 0x9 ) continue;
 
         p_desc = descs_get_desc( p_descs, k );
         if ( p_desc == NULL ) continue; /* This shouldn't happen */
@@ -1653,6 +1650,7 @@ static void DeleteProgram( uint16_t i_sid, uint16_t i_pid )
 {
     sid_t *p_sid;
     uint8_t *p_pmt;
+    uint8_t *p_desc;
 
     UnselectPMT( i_sid, i_pid );
 
@@ -1678,7 +1676,6 @@ static void DeleteProgram( uint16_t i_sid, uint16_t i_pid )
         if ( b_enable_ecm )
         {
             j = 0;
-            uint8_t *p_desc;
 
             while ((p_desc = descs_get_desc( pmt_get_descs( p_pmt ), j++ )) != NULL)
             {
@@ -1697,6 +1694,18 @@ static void DeleteProgram( uint16_t i_sid, uint16_t i_pid )
 
             if ( PIDWouldBeSelected( p_es ) )
                 UnselectPID( i_sid, i_pid );
+
+            if ( b_enable_ecm )
+            {
+                uint8_t k = 0;
+
+                while ((p_desc = descs_get_desc( pmtn_get_descs( p_es ), k++ )) != NULL)
+                {
+                    if ( desc_get_tag( p_desc ) != 0x09 || !desc09_validate( p_desc ) )
+                        continue;
+                    UnselectPID( i_sid, desc09_get_pid( p_desc ) );
+                }
+            }
         }
 
         free( p_pmt );
@@ -2106,6 +2115,7 @@ static void HandlePMT( uint16_t i_pid, uint8_t *p_pmt, mtime_t i_dts )
     uint8_t *p_es;
     uint8_t *p_desc;
     uint16_t j;
+    uint16_t k;
 
     p_sid = FindSID( i_sid );
     if ( p_sid == NULL )
@@ -2194,6 +2204,17 @@ static void HandlePMT( uint16_t i_pid, uint8_t *p_pmt, mtime_t i_dts )
         if ( PIDWouldBeSelected( p_es ) )
             SelectPID( i_sid, i_pid );
         p_pids[i_pid].b_pes = PIDCarriesPES( p_es );
+
+        if ( b_enable_ecm )
+        {
+            k = 0;
+            while ( (p_desc = descs_get_desc( pmtn_get_descs( p_es ), k++ )) != NULL )
+            {
+                if ( desc_get_tag( p_desc ) != 0x09 || !desc09_validate( p_desc ) )
+                    continue;
+                SelectPID( i_sid, desc09_get_pid( p_desc ) );
+            }
+        }
     }
 
     if ( p_sid->p_current_pmt != NULL )
@@ -2230,6 +2251,23 @@ static void HandlePMT( uint16_t i_pid, uint8_t *p_pmt, mtime_t i_dts )
                 if ( pmt_find_es( p_pmt, i_current_pid ) == NULL )
                     UnselectPID( i_sid, i_current_pid );
             }
+
+            if ( b_enable_ecm )
+            {
+                k = 0;
+                uint16_t f = 0;
+                uint8_t *p_pmt_es;
+                while ((p_desc = descs_get_desc( pmtn_get_descs( p_es ), k++ )) != NULL)
+                {
+                    if ( desc_get_tag( p_desc ) != 0x09 || !desc09_validate( p_desc ) )
+                        continue;
+                    while ( (p_pmt_es = pmt_get_es( p_pmt, f++ )) != NULL )
+                    {
+                        if ( ca_desc_find( pmtn_get_descs( p_pmt_es ) + DESCS_HEADER_SIZE, descs_get_length( pmtn_get_descs( p_pmt_es ) ), desc09_get_pid( p_desc ) ) == NULL )
+                            UnselectPID( i_sid, desc09_get_pid( p_desc ) );
+                    }
+                }
+            }
         }
 
         free( p_sid->p_current_pmt );
