@@ -1,8 +1,7 @@
 /*****************************************************************************
- * dvb.c
+ * dvb.c: linux-dvb input for DVBlast
  *****************************************************************************
- * Copyright (C) 2008-2009 VideoLAN
- * $Id: dvb.c 72 2009-11-23 21:52:43Z massiot $
+ * Copyright (C) 2008-2010 VideoLAN
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -23,7 +22,9 @@
 
 #include <stdlib.h>
 #include <stdint.h>
+#include <stdbool.h>
 #include <stdio.h>
+#include <unistd.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -47,29 +48,29 @@
 #include "en50221.h"
 #include "comm.h"
 
+#include <bitstream/common.h>
+
 /*****************************************************************************
  * Local declarations
  *****************************************************************************/
-#define FRONTEND_LOCK_TIMEOUT 30000000 /* 30 s */
-#define COUNTER_WRAP 200 /* we make 200 read calls per second */
+#define DVR_READ_TIMEOUT 30000000 /* 30 s */
+#define CA_POLL_PERIOD 100000 /* 100 ms */
 #define MAX_READ_ONCE 50
 #define DVR_BUFFER_SIZE 40*188*1024 /* bytes */
 
 static int i_frontend, i_dvr;
 static fe_status_t i_last_status;
 static mtime_t i_frontend_timeout;
-static unsigned int i_read_once = 1;
-static unsigned int i_read_counter = 0;
-static mtime_t i_last_counter = 0;
+static mtime_t i_last_packet = 0;
 static mtime_t i_ca_next_event = 0;
-mtime_t i_ca_timeout = 0;
+static block_t *p_freelist = NULL;
 
 /*****************************************************************************
  * Local prototypes
  *****************************************************************************/
 static block_t *DVRRead( void );
 static void FrontendPoll( void );
-static void FrontendSet( void );
+static void FrontendSet( bool b_reset );
 
 /*****************************************************************************
  * dvb_Open
@@ -80,6 +81,8 @@ void dvb_Open( void )
 
     msg_Dbg( NULL, "using linux-dvb API version %d", DVB_API_VERSION );
 
+    i_wallclock = mdate();
+
     sprintf( psz_tmp, "/dev/dvb/adapter%d/frontend%d", i_adapter, i_fenum );
     if( (i_frontend = open(psz_tmp, O_RDWR | O_NONBLOCK)) < 0 )
     {
@@ -88,23 +91,17 @@ void dvb_Open( void )
         exit(1);
     }
 
-    FrontendSet();
+    FrontendSet(true);
 
     sprintf( psz_tmp, "/dev/dvb/adapter%d/dvr%d", i_adapter, i_fenum );
 
-    if( (i_dvr = open(psz_tmp, O_RDONLY)) < 0 )
+    if( (i_dvr = open(psz_tmp, O_RDONLY | O_NONBLOCK)) < 0 )
     {
         msg_Err( NULL, "opening device %s failed (%s)", psz_tmp,
                  strerror(errno) );
         exit(1);
     }
 
-    if( fcntl( i_dvr, F_SETFL, O_NONBLOCK ) == -1 )
-    {
-        msg_Warn( NULL, "couldn't set %s non-blocking mode (%s)", psz_tmp,
-                  strerror(errno) );
-    }
-
     if ( ioctl( i_dvr, DMX_SET_BUFFER_SIZE, DVR_BUFFER_SIZE ) < 0 )
     {
         msg_Warn( NULL, "couldn't set %s buffer size (%s)", psz_tmp,
@@ -112,16 +109,25 @@ void dvb_Open( void )
     }
 
     en50221_Init();
-    i_ca_next_event = mdate() + i_ca_timeout;
+    i_ca_next_event = mdate() + CA_POLL_PERIOD;
+}
+
+/*****************************************************************************
+ * dvb_Reset
+ *****************************************************************************/
+void dvb_Reset( void )
+{
+    FrontendSet(true);
 }
 
 /*****************************************************************************
  * dvb_Read
  *****************************************************************************/
-block_t *dvb_Read( void )
+block_t *dvb_Read( mtime_t i_poll_timeout )
 {
-    struct pollfd ufds[3];
+    struct pollfd ufds[4];
     int i_ret, i_nb_fd = 2;
+    block_t *p_blocks = NULL;
 
     memset( ufds, 0, sizeof(ufds) );
     ufds[0].fd = i_dvr;
@@ -130,12 +136,20 @@ block_t *dvb_Read( void )
     ufds[1].events = POLLERR | POLLPRI;
     if ( i_comm_fd != -1 )
     {
-        ufds[2].fd = i_comm_fd;
-        ufds[2].events = POLLIN;
-        i_nb_fd = 3;
+        ufds[i_nb_fd].fd = i_comm_fd;
+        ufds[i_nb_fd].events = POLLIN;
+        i_nb_fd++;
+    }
+    if ( i_ca_handle && i_ca_type == CA_CI_LINK )
+    {
+        ufds[i_nb_fd].fd = i_ca_handle;
+        ufds[i_nb_fd].events = POLLIN;
+        i_nb_fd++;
     }
 
-    i_ret = poll( ufds, i_nb_fd, 100 );
+    i_ret = poll( ufds, i_nb_fd, (i_poll_timeout + 999) / 1000 );
+
+    i_wallclock = mdate();
 
     if ( i_ret < 0 )
     {
@@ -144,29 +158,61 @@ block_t *dvb_Read( void )
         return NULL;
     }
 
-    if ( i_ca_handle && i_ca_type == CA_CI_LINK
-          && mdate() > i_ca_next_event )
-    {
-        en50221_Poll();
-        i_ca_next_event = mdate() + i_ca_timeout;
-    }
-
     if ( ufds[1].revents )
         FrontendPoll();
 
-    if ( i_frontend_timeout && mdate() > i_frontend_timeout )
+    if ( ufds[0].revents )
     {
+        p_blocks = DVRRead();
+        i_wallclock = mdate();
+    }
+
+    if ( p_blocks != NULL )
+        i_last_packet = i_wallclock;
+    else if ( !i_frontend_timeout
+                && i_wallclock > i_last_packet + DVR_READ_TIMEOUT )
+    {
+        msg_Warn( NULL, "no DVR output, resetting" );
+        FrontendSet(false);
+        en50221_Reset();
+    }
+
+    if ( i_ca_handle && i_ca_type == CA_CI_LINK )
+    {
+        if ( ufds[i_nb_fd - 1].revents )
+        {
+            en50221_Read();
+            i_ca_next_event = i_wallclock + CA_POLL_PERIOD;
+        }
+        else if ( i_wallclock > i_ca_next_event )
+        {
+            en50221_Poll();
+            i_ca_next_event = i_wallclock + CA_POLL_PERIOD;
+        }
+    }
+
+    if ( i_frontend_timeout && i_wallclock > i_frontend_timeout )
+    {
+        if ( i_quit_timeout_duration )
+        {
+            msg_Err( NULL, "no lock" );
+            switch (i_print_type) {
+            case PRINT_XML:
+                printf("</TS>\n");
+                break;
+            default:
+                break;
+            }
+            exit(EXIT_STATUS_FRONTEND_TIMEOUT);
+        }
         msg_Warn( NULL, "no lock, tuning again" );
-        FrontendSet();
+        FrontendSet(false);
     }
 
     if ( i_comm_fd != -1 && ufds[2].revents )
         comm_Read();
 
-    if ( ufds[0].revents )
-        return DVRRead();
-
-    return NULL;
+    return p_blocks;
 }
 
 /*****************************************************************************
@@ -175,18 +221,18 @@ block_t *dvb_Read( void )
 static block_t *DVRRead( void )
 {
     int i, i_len;
-    block_t *p_ts, **pp_current = &p_ts;
-    struct iovec p_iov[i_read_once];
+    block_t *p_ts = p_freelist, **pp_current = &p_ts;
+    struct iovec p_iov[MAX_READ_ONCE];
 
-    for ( i = 0; i < i_read_once; i++ )
+    for ( i = 0; i < MAX_READ_ONCE; i++ )
     {
-        *pp_current = block_New();
+        if ( (*pp_current) == NULL ) *pp_current = block_New();
         p_iov[i].iov_base = (*pp_current)->p_ts;
         p_iov[i].iov_len = TS_SIZE;
         pp_current = &(*pp_current)->p_next;
     }
 
-    if ( (i_len = readv(i_dvr, p_iov, i_read_once)) < 0 )
+    if ( (i_len = readv(i_dvr, p_iov, MAX_READ_ONCE)) < 0 )
     {
         msg_Err( NULL, "couldn't read from DVR device (%s)",
                  strerror(errno) );
@@ -194,8 +240,6 @@ static block_t *DVRRead( void )
     }
     i_len /= TS_SIZE;
 
-    //msg_Err( NULL, "Meuuh %d %d", i_len, i_read_once );
-
     pp_current = &p_ts;
     while ( i_len && *pp_current )
     {
@@ -203,30 +247,9 @@ static block_t *DVRRead( void )
         i_len--;
     }
 
-    block_DeleteChain( *pp_current );
+    p_freelist = *pp_current;
     *pp_current = NULL;
 
-    i_read_counter++;
-    if ( i_read_counter >= COUNTER_WRAP )
-    {
-        mtime_t i_current_date = mdate();
-
-        if ( i_last_counter )
-        {
-            /* Adjust the buffer size to keep the read() calls frequency
-             * at a certain limit */
-            i_read_once = (mtime_t)i_read_once * 1000000LL
-                           / (i_current_date - i_last_counter);
-            if ( i_read_once < 1 )
-                i_read_once = 1;
-            if ( i_read_once > MAX_READ_ONCE )
-                i_read_once = MAX_READ_ONCE;
-        }
-
-        i_read_counter = 0;
-        i_last_counter = i_current_date;
-    }
-
     return p_ts;
 }
 
@@ -345,8 +368,19 @@ static void FrontendPoll( void )
             IF_UP( FE_HAS_LOCK )
             {
                 int32_t i_value = 0;
-                msg_Dbg( NULL, "frontend has acquired lock" );
+                msg_Info( NULL, "frontend has acquired lock" );
+                switch (i_print_type) {
+                case PRINT_XML:
+                    printf("<STATUS type=\"lock\" status=\"1\" />\n");
+                    break;
+                default:
+                    printf("frontend has acquired lock\n" );
+                }
                 i_frontend_timeout = 0;
+                i_last_packet = i_wallclock;
+
+                if ( i_quit_timeout_duration && !i_quit_timeout )
+                    i_quit_timeout = i_wallclock + i_quit_timeout_duration;
 
                 /* Read some statistics */
                 if( ioctl( i_frontend, FE_READ_BER, &i_value ) >= 0 )
@@ -359,14 +393,21 @@ static void FrontendPoll( void )
             else
             {
                 msg_Dbg( NULL, "frontend has lost lock" );
-                i_frontend_timeout = mdate() + FRONTEND_LOCK_TIMEOUT;
+                switch (i_print_type) {
+                case PRINT_XML:
+                    printf("<STATUS type=\"lock\" status=\"0\"/>\n");
+                    break;
+                default:
+                    printf("frontend has lost lock\n" );
+                }
+                i_frontend_timeout = i_wallclock + i_frontend_timeout_duration;
             }
 
             IF_UP( FE_REINIT )
             {
                 /* The frontend was reinited. */
                 msg_Warn( NULL, "reiniting frontend");
-                FrontendSet();
+                FrontendSet(true);
             }
         }
 #undef IF_UP
@@ -484,13 +525,13 @@ static int FrontendDoDiseqc(void)
         if( ioctl( i_frontend, FE_DISEQC_SEND_BURST,
                    i_satnum == 0xB ? SEC_MINI_B : SEC_MINI_A ) < 0 )
         {
-            msg_Err( NULL, "ioctl FE_SEND_BURST failed (%m)", strerror(errno) );
+            msg_Err( NULL, "ioctl FE_SEND_BURST failed (%s)", strerror(errno) );
             exit(1);
         }
         msleep(100000); /* ... */
     }
 
-    else if ( ioctl( i_frontend, FE_SET_TONE, fe_tone ) < 0 )
+    if ( ioctl( i_frontend, FE_SET_TONE, fe_tone ) < 0 )
     {
         msg_Err( NULL, "FE_SET_TONE failed (%s)", strerror(errno) );
         exit(1);
@@ -508,8 +549,51 @@ static int FrontendDoDiseqc(void)
 #if DVB_API_VERSION >= 5
 
 /*****************************************************************************
- * GetModulation : helper function for both APIs
+ * Helper functions for S2API
  *****************************************************************************/
+static fe_spectral_inversion_t GetInversion(void)
+{
+    switch ( i_inversion )
+    {
+        case 0:  return INVERSION_OFF;
+        case 1:  return INVERSION_ON;
+        default:
+            msg_Warn( NULL, "invalid inversion %d", i_inversion );
+        case -1: return INVERSION_AUTO;
+    }
+}
+
+static fe_code_rate_t GetFEC(fe_caps_t fe_caps, int i_fec_value)
+{
+#define GET_FEC_INNER(fec, val)                                             \
+    if ( (fe_caps & FE_CAN_##fec) && (i_fec_value == val) )                 \
+       return fec;
+
+    GET_FEC_INNER(FEC_AUTO, 999);
+    GET_FEC_INNER(FEC_AUTO, -1);
+    if (i_fec_value == 0)
+        return FEC_NONE;
+    GET_FEC_INNER(FEC_1_2, 12);
+    GET_FEC_INNER(FEC_2_3, 23);
+    GET_FEC_INNER(FEC_3_4, 34);
+    if (i_fec_value == 35)
+        return FEC_3_5;
+    GET_FEC_INNER(FEC_4_5, 45);
+    GET_FEC_INNER(FEC_5_6, 56);
+    GET_FEC_INNER(FEC_6_7, 67);
+    GET_FEC_INNER(FEC_7_8, 78);
+    GET_FEC_INNER(FEC_8_9, 89);
+    if (i_fec_value == 910)
+        return FEC_9_10;
+
+#undef GET_FEC_INNER
+    msg_Warn(NULL, "invalid FEC %d", i_fec_value );
+    return FEC_AUTO;
+}
+
+#define GetFECInner(caps) GetFEC(caps, i_fec)
+#define GetFECLP(caps) GetFEC(caps, i_fec_lp)
+
 static fe_modulation_t GetModulation(void)
 {
 #define GET_MODULATION( mod )                                               \
@@ -525,18 +609,151 @@ static fe_modulation_t GetModulation(void)
     GET_MODULATION(QAM_AUTO);
     GET_MODULATION(VSB_8);
     GET_MODULATION(VSB_16);
-#if DVB_API_VERSION >= 5
     GET_MODULATION(PSK_8);
     GET_MODULATION(APSK_16);
     GET_MODULATION(APSK_32);
     GET_MODULATION(DQPSK);
-#endif
 
 #undef GET_MODULATION
     msg_Err( NULL, "invalid modulation %s", psz_modulation );
     exit(1);
 }
 
+static fe_pilot_t GetPilot(void)
+{
+    switch ( i_pilot )
+    {
+        case 0:  return PILOT_OFF;
+        case 1:  return PILOT_ON;
+        default:
+            msg_Warn( NULL, "invalid pilot %d", i_pilot );
+        case -1: return PILOT_AUTO;
+    }
+}
+
+static fe_rolloff_t GetRollOff(void)
+{
+    switch ( i_rolloff )
+    {
+        case -1:
+        case  0: return ROLLOFF_AUTO;
+        case 20: return ROLLOFF_20;
+        case 25: return ROLLOFF_25;
+        default:
+            msg_Warn( NULL, "invalid rolloff %d", i_rolloff );
+        case 35: return ROLLOFF_35;
+    }
+}
+
+static fe_guard_interval_t GetGuard(void)
+{
+    switch ( i_guard )
+    {
+        case 32: return GUARD_INTERVAL_1_32;
+        case 16: return GUARD_INTERVAL_1_16;
+        case  8: return GUARD_INTERVAL_1_8;
+        case  4: return GUARD_INTERVAL_1_4;
+        default:
+            msg_Warn( NULL, "invalid guard interval %d", i_guard );
+        case -1:
+        case  0: return GUARD_INTERVAL_AUTO;
+    }
+}
+
+static fe_transmit_mode_t GetTransmission(void)
+{
+    switch ( i_transmission )
+    {
+        case 2: return TRANSMISSION_MODE_2K;
+        case 8: return TRANSMISSION_MODE_8K;
+#ifdef TRANSMISSION_MODE_4K
+        case 4: return TRANSMISSION_MODE_4K;
+#endif
+        default:
+            msg_Warn( NULL, "invalid tranmission mode %d", i_transmission );
+        case -1:
+        case 0: return TRANSMISSION_MODE_AUTO;
+    }
+}
+
+static fe_hierarchy_t GetHierarchy(void)
+{
+    switch ( i_hierarchy )
+    {
+        case 0: return HIERARCHY_NONE;
+        case 1: return HIERARCHY_1;
+        case 2: return HIERARCHY_2;
+        case 4: return HIERARCHY_4;
+        default:
+            msg_Warn( NULL, "invalid intramission mode %d", i_transmission );
+        case -1: return HIERARCHY_AUTO;
+    }
+}
+
+/*****************************************************************************
+ * FrontendInfo : Print frontend info
+ *****************************************************************************/
+static const char *GetFrontendTypeName( fe_type_t type )
+{
+    switch(type)
+    {
+        case FE_QPSK: return "QPSK (DVB-S/S2)";
+        case FE_QAM:  return "QAM  (DVB-C)";
+        case FE_OFDM: return "OFDM (DVB-T)";
+        case FE_ATSC: return "ATSC";
+        default: return "unknown";
+    }
+}
+
+static void FrontendInfo( struct dvb_frontend_info info )
+{
+    msg_Dbg( NULL, "Frontend \"%s\" type \"%s\" supports:",
+             info.name, GetFrontendTypeName(info.type) );
+    msg_Dbg( NULL, " frequency min: %d, max: %d, stepsize: %d, tolerance: %d",
+             info.frequency_min, info.frequency_max,
+             info.frequency_stepsize, info.frequency_tolerance );
+    msg_Dbg( NULL, " symbolrate min: %d, max: %d, tolerance: %d",
+             info.symbol_rate_min, info.symbol_rate_max, info.symbol_rate_tolerance);
+    msg_Dbg( NULL, " capabilities:" );
+
+#define FRONTEND_INFO(caps,val,msg)                                             \
+    if ( caps & val )                                                       \
+        msg_Dbg( NULL, "  %s", msg );
+
+    FRONTEND_INFO( info.caps, FE_IS_STUPID, "FE_IS_STUPID" )
+    FRONTEND_INFO( info.caps, FE_CAN_INVERSION_AUTO, "INVERSION_AUTO" )
+    FRONTEND_INFO( info.caps, FE_CAN_FEC_1_2, "FEC_1_2" )
+    FRONTEND_INFO( info.caps, FE_CAN_FEC_2_3, "FEC_2_3" )
+    FRONTEND_INFO( info.caps, FE_CAN_FEC_3_4, "FEC_3_4" )
+    FRONTEND_INFO( info.caps, FE_CAN_FEC_4_5, "FEC_4_5" )
+    FRONTEND_INFO( info.caps, FE_CAN_FEC_5_6, "FEC_5_6" )
+    FRONTEND_INFO( info.caps, FE_CAN_FEC_6_7, "FEC_6_7" )
+    FRONTEND_INFO( info.caps, FE_CAN_FEC_7_8, "FEC_7_8" )
+    FRONTEND_INFO( info.caps, FE_CAN_FEC_8_9, "FEC_8_9" )
+    FRONTEND_INFO( info.caps, FE_CAN_FEC_AUTO,"FEC_AUTO")
+    FRONTEND_INFO( info.caps, FE_CAN_QPSK,   "QPSK" )
+    FRONTEND_INFO( info.caps, FE_CAN_QAM_16, "QAM_16" )
+    FRONTEND_INFO( info.caps, FE_CAN_QAM_32, "QAM_32" )
+    FRONTEND_INFO( info.caps, FE_CAN_QAM_64, "QAM_64" )
+    FRONTEND_INFO( info.caps, FE_CAN_QAM_128,"QAM_128")
+    FRONTEND_INFO( info.caps, FE_CAN_QAM_256,"QAM_256")
+    FRONTEND_INFO( info.caps, FE_CAN_QAM_AUTO,"QAM_AUTO" )
+    FRONTEND_INFO( info.caps, FE_CAN_TRANSMISSION_MODE_AUTO, "TRANSMISSION_MODE_AUTO" )
+    FRONTEND_INFO( info.caps, FE_CAN_BANDWIDTH_AUTO, "BANDWIDTH_AUTO" )
+    FRONTEND_INFO( info.caps, FE_CAN_GUARD_INTERVAL_AUTO, "GUARD_INTERVAL_AUTO" )
+    FRONTEND_INFO( info.caps, FE_CAN_HIERARCHY_AUTO, "HIERARCHY_AUTO" )
+    FRONTEND_INFO( info.caps, FE_CAN_8VSB, "8VSB" )
+    FRONTEND_INFO( info.caps, FE_CAN_16VSB,"16VSB" )
+    FRONTEND_INFO( info.caps, FE_HAS_EXTENDED_CAPS, "EXTENDED_CAPS" )
+#if DVBAPI_VERSION >= 501
+    FRONTEND_INFO( info.caps, FE_CAN_2G_MODULATION, "2G_MODULATION" )
+#endif
+    FRONTEND_INFO( info.caps, FE_NEEDS_BENDING, "NEEDS_BENDING" )
+    FRONTEND_INFO( info.caps, FE_CAN_RECOVER, "FE_CAN_RECOVER" )
+    FRONTEND_INFO( info.caps, FE_CAN_MUTE_TS, "FE_CAN_MUTE_TS" )
+#undef FRONTEND_INFO
+}
+
 /*****************************************************************************
  * FrontendSet
  *****************************************************************************/
@@ -602,13 +819,41 @@ static struct dtv_properties dvbt_cmdseq = {
     .props = dvbt_cmdargs
 };
 
+static struct dtv_property atsc_cmdargs[] = {
+    { .cmd = DTV_FREQUENCY, .u.data = 0 },
+    { .cmd = DTV_MODULATION, .u.data = QAM_AUTO },
+    { .cmd = DTV_INVERSION, .u.data = INVERSION_AUTO },
+    { .cmd = DTV_DELIVERY_SYSTEM, .u.data = SYS_ATSC },
+    { .cmd = DTV_TUNE },
+};
+static struct dtv_properties atsc_cmdseq = {
+    .num = sizeof(atsc_cmdargs)/sizeof(struct dtv_property),
+    .props = atsc_cmdargs
+};
+
 #define FREQUENCY 0
 #define MODULATION 1
 #define INVERSION 2
 #define SYMBOL_RATE 3
 #define BANDWIDTH 3
+#define FEC_INNER 4
+#define FEC_LP 5
+#define GUARD 6
+#define PILOT 6
+#define TRANSMISSION 7
+#define ROLLOFF 7
+#define HIERARCHY 8
 
-static void FrontendSet( void )
+struct dtv_property pclear[] = {
+    { .cmd = DTV_CLEAR },
+};
+
+struct dtv_properties cmdclear = {
+    .num = 1,
+    .props = pclear
+};
+
+static void FrontendSet( bool b_init )
 {
     struct dvb_frontend_info info;
     struct dtv_properties *p;
@@ -619,29 +864,48 @@ static void FrontendSet( void )
         exit(1);
     }
 
+    if ( b_init )
+        FrontendInfo( info );
+
+    /* Clear frontend commands */
+    if ( ioctl( i_frontend, FE_SET_PROPERTY, &cmdclear ) < 0 )
+    {
+        msg_Err( NULL, "Unable to clear frontend" );
+        exit(1);
+    }
+
     switch ( info.type )
     {
     case FE_OFDM:
         p = &dvbt_cmdseq;
         p->props[FREQUENCY].u.data = i_frequency;
+        p->props[INVERSION].u.data = GetInversion();
         if ( psz_modulation != NULL )
             p->props[MODULATION].u.data = GetModulation();
         p->props[BANDWIDTH].u.data = i_bandwidth * 1000000;
+        p->props[FEC_INNER].u.data = GetFECInner(info.caps);
+        p->props[FEC_LP].u.data = GetFECLP(info.caps);
+        p->props[GUARD].u.data = GetGuard();
+        p->props[TRANSMISSION].u.data = GetTransmission();
+        p->props[HIERARCHY].u.data = GetHierarchy();
 
-        msg_Dbg( NULL, "tuning OFDM frontend to f=%d bandwidth=%d modulation=%s",
-                 i_frequency, i_bandwidth,
-                 psz_modulation == NULL ? "qam_auto" : psz_modulation );
+        msg_Dbg( NULL, "tuning OFDM frontend to f=%d bandwidth=%d inversion=%d fec_hp=%d fec_lp=%d hierarchy=%d modulation=%s guard=%d transmission=%d",
+                 i_frequency, i_bandwidth, i_inversion, i_fec, i_fec_lp,
+                 i_hierarchy,
+                 psz_modulation == NULL ? "qam_auto" : psz_modulation,
+                 i_guard, i_transmission );
         break;
 
     case FE_QAM:
         p = &dvbc_cmdseq;
         p->props[FREQUENCY].u.data = i_frequency;
+        p->props[INVERSION].u.data = GetInversion();
         if ( psz_modulation != NULL )
             p->props[MODULATION].u.data = GetModulation();
         p->props[SYMBOL_RATE].u.data = i_srate;
 
-        msg_Dbg( NULL, "tuning QAM frontend to f=%d srate=%d modulation=%s",
-                 i_frequency, i_srate,
+        msg_Dbg( NULL, "tuning QAM frontend to f=%d srate=%d inversion=%d modulation=%s",
+                 i_frequency, i_srate, i_inversion,
                  psz_modulation == NULL ? "qam_auto" : psz_modulation );
         break;
 
@@ -650,16 +914,32 @@ static void FrontendSet( void )
         {
             p = &dvbs2_cmdseq;
             p->props[MODULATION].u.data = GetModulation();
+            p->props[PILOT].u.data = GetPilot();
+            p->props[ROLLOFF].u.data = GetRollOff();
         }
         else
             p = &dvbs_cmdseq;
 
+        p->props[INVERSION].u.data = GetInversion();
         p->props[SYMBOL_RATE].u.data = i_srate;
+        p->props[FEC_INNER].u.data = GetFECInner(info.caps);
         p->props[FREQUENCY].u.data = FrontendDoDiseqc();
 
-        msg_Dbg( NULL, "tuning QPSK frontend to f=%d srate=%d modulation=%s",
-                 i_frequency, i_srate,
-                 psz_modulation == NULL ? "legacy" : psz_modulation );
+        msg_Dbg( NULL, "tuning QPSK frontend to f=%d srate=%d inversion=%d fec=%d rolloff=%d modulation=%s pilot=%d",
+                 i_frequency, i_srate, i_inversion, i_fec, i_rolloff,
+                 psz_modulation == NULL ? "legacy" : psz_modulation, i_pilot );
+        break;
+
+    case FE_ATSC:
+        p = &atsc_cmdseq;
+        p->props[FREQUENCY].u.data = i_frequency;
+        p->props[INVERSION].u.data = GetInversion();
+        if ( psz_modulation != NULL )
+            p->props[MODULATION].u.data = GetModulation();
+
+        msg_Dbg( NULL, "tuning ATSC frontend to f=%d inversion=%d modulation=%s",
+                 i_frequency, i_inversion,
+                 psz_modulation == NULL ? "qam_auto" : psz_modulation );
         break;
 
     default:
@@ -684,12 +964,15 @@ static void FrontendSet( void )
     }
 
     i_last_status = 0;
-    i_frontend_timeout = mdate() + FRONTEND_LOCK_TIMEOUT;
+    i_frontend_timeout = i_wallclock + i_frontend_timeout_duration;
 }
 
 #else /* !S2API */
 
-static void FrontendSet( void )
+#warning "You are trying to compile DVBlast with an outdated linux-dvb interface."
+#warning "DVBlast will be very limited and some options will have no effect."
+
+static void FrontendSet( bool b_init )
 {
     struct dvb_frontend_info info;
     struct dvb_frontend_parameters fep;
@@ -732,8 +1015,8 @@ static void FrontendSet( void )
         fep.u.qam.fec_inner = FEC_AUTO;
         fep.u.qam.modulation = QAM_AUTO;
 
-        msg_Dbg( NULL, "tuning QAM frontend to f=%d, srate=%d", i_frequency,
-                 i_srate );
+        msg_Dbg( NULL, "tuning QAM frontend to f=%d, srate=%d",
+                 i_frequency, i_srate );
         break;
 
     case FE_QPSK:
@@ -778,7 +1061,7 @@ static void FrontendSet( void )
     }
 
     i_last_status = 0;
-    i_frontend_timeout = mdate() + FRONTEND_LOCK_TIMEOUT;
+    i_frontend_timeout = i_wallclock + i_frontend_timeout_duration;
 }
 
 #endif /* S2API */
@@ -795,6 +1078,7 @@ uint8_t dvb_FrontendStatus( uint8_t *p_answer, ssize_t *pi_size )
         msg_Err( NULL, "ioctl FE_GET_INFO failed (%s)", strerror(errno) );
         return RET_ERR;
     }
+
     if ( ioctl( i_frontend, FE_READ_STATUS, &p_ret->i_status ) < 0 )
     {
         msg_Err( NULL, "ioctl FE_READ_STATUS failed (%s)", strerror(errno) );
